package de.hft.stuttgart.citydoctor2.zip;

import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import de.hft.stuttgart.citydoctor2.utils.ArchivePacker;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.*;

import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;

import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

public class CityGmlZipArchive implements Serializable {

    private static final Logger logger = LogManager.getLogger(CityGmlZipArchive.class);

    @Serial
    private static final long serialVersionUID = 2168389511043362615L;

    private List<CityGmlZipEntry> entries;
    private final Path archivePath;
    private final String archiveNameRE;


    public static CityGmlZipArchive register(String zipFile) {
        ArrayList<CityGmlZipEntry> archiveEntries = new ArrayList<>();
        CityGmlZipArchive cgmlArchive = new CityGmlZipArchive(Path.of(zipFile));
        try (ZipInputStream zis = new ZipInputStream(new FileInputStream(zipFile))) {
            ZipEntry ze;
            while ((ze = zis.getNextEntry()) != null )
            {
                if (ze.isDirectory()) {
                    continue;
                }
                if (ze.getName().endsWith(".gml")) {
                    archiveEntries.add(CityGmlZipEntry.register(ze, cgmlArchive));
                }
            }
            cgmlArchive.setEntries(archiveEntries);
            return cgmlArchive;
        } catch (IOException e) {
            logger.error("Encountered an IOError while registering CityGmlArchive", e);
            return null;
        }
    }

    public void mountArchive(ParserConfiguration config){
        try (ZipFile zip = new ZipFile(archivePath.toFile())) {
            for (CityGmlZipEntry entry : entries){
                entry.loadEntry(config);
            }
        } catch (IOException e) {
            logger.error(e);
        }
    }

    private CityGmlZipArchive(Path archivePath){
        this.archivePath = archivePath;
        this.archiveNameRE = archivePath.getFileName().toString().replace(".zip","") + File.separator;
    }

    private void setEntries(List<CityGmlZipEntry> entries) {
        this.entries = entries;
        entries.forEach(e -> e.setArchive(this));
    }

    public void exportToZipFile(String path)  {
        ArchivePacker.packArchive(path, this);
    }


    public CityGmlZipEntry getEntry(String fileName) {
        fileName = stripArchivePath(fileName);
        for(CityGmlZipEntry entry : entries){
            String entryName = stripArchivePath(entry.getFileName());
            if(entryName.equals(fileName)){
                return entry;
            }
        }
        return null;
    }

    private String stripArchivePath(String path) {
        Path systemPath = Path.of(path);
        return systemPath.toString().replace(archiveNameRE, "");
    }

    public Path getArchivePath() {
        return archivePath;
    }

    public List<CityGmlZipEntry> getEntries() {
        return entries;
    }
}
